{
 /***************************************************************************
                               ExtMsgDlg.pp
                               ----------
 ***************************************************************************/

 *****************************************************************************
 *                                                                           *
 *  See the file COPYING.modifiedLGPL.txt, included in this distribution,    *
 *  for details about the copyright.                                         *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 *****************************************************************************

 This unit provides several new overloaded MessageDlg() functions.

 The extensions are:
  - Add a checkbox to the dialog form (e.g. for "Don't show this message again"
   functionality)
  - Currently the CheckBoxChecked is used as an out parameter only.
    The checkbox is not cheked upon showing the dialog.
    (This behaviour might change in the future, if and when deemed necessry for my
    own purposes
  - A global variable DefaultMsgDlgPosition controls where the dialogs are placed
    on the screen:
    - dpScreenCenter: like poScreenCenter
    - dpMainFormCenter: centered over Application.MainForm, but then, if necessary,
      repositionded to keep in view on the desktop
    - dpCallerFormCenter: if the optional parameter CallerForm is specified, then
      the dialog is centered over the calling form. If the CallerForm is not specified (= nil)
      then the dialog is centered over the MainForm. Repositioning to keep in view
      on the desktop is done if necessary. This is the default value.
   - The functions are declared with the overload keyword, so they can be overloaded,
     even in another unit
}


unit ExtMsgDlg;

{$mode objfpc}{$H+}

interface

uses
  LCLType, Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs, Buttons, StdCtrls,
  ExtCtrls;

type
  TDlgPosition = (dpScreenCenter, dpMainFormCenter, dpCallerFormCenter);

var
  DefaultMsgDlgPosition: TDlgPosition;

const
  HeightCorrection: Integer = 25;  //Estimated guess for correction of Caption bar, feel free to alter
  //DoTranslateCaptions: Boolean = True;


{
  This function is needed because pressing Escape in the dialog
  will return -1 as ModalResult. The ModalEscapeValue() function translates
  this to the appropriate mrCancel/mrNone,mrAbort etc. depending on the buttons
  that are present in the dialog.
}
function ModalEscapeValue(Buttons : TMsgDlgButtons) : TModalResult;

//Helper procedures that may come in handy, so make it public
procedure CenterFormOver(AForm, CallerForm: TCustomForm);
procedure KeepInView(AForm: TCustomForm);

{
  These constants translate ModalResults form CreateMessageDialog's ShowModal ????

const
  DialogResult : Array[mrNone..mrYesToAll] of Longint = (
    -1, idButtonOK, idButtonCancel, idButtonAbort, idButtonRetry,
    idButtonIgnore, idButtonYes,idButtonNo, idButtonAll, idButtonNoToAll,
    idButtonYesToAll);

}



{

function MessageDlg(const aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; CallerForm: TCustomForm = nil): Integer;
function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; CallerForm: TCustomForm = nil): Integer;
function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; DefaultButton: TMsgDlgBtn; CallerForm: TCustomForm = nil): Integer;
function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; const HelpKeyword: string; CallerForm: TCustomForm = nil): Integer;
function MessageDlgPos(const aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;


}

function MessageDlg(const aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; CallerForm: TCustomForm = nil): Integer;
function MessageDlg(aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; DefaultButton: TMsgDlgBtn; CallerForm: TCustomForm = nil): Integer;
function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; CallerForm: TCustomForm = nil): Integer;
function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; DefaultButton: TMsgDlgBtn; CallerForm: TCustomForm = nil): Integer;
function MessageDlgPos(const aMsg: string; DlgType: TMsgDlgType;
            Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;


function ExtMessageDlg(const AMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckBoxCaption: String; out CheckBoxChecked: Boolean;
  HelpCtx: Longint; CallerForm: TCustomForm = nil): TModalResult; overload;

function ExtMessageDlg(const ACaption, AMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckBoxCaption: String; out CheckBoxChecked: Boolean;
  HelpCtx: Longint; CallerForm: TCustomForm = nil): TModalResult; overload;

function ExtMessageDlgPos(const ACaption, AMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckBoxCaption: String; out CheckBoxChecked: Boolean;
  HelpCtx: Longint; X, Y: LongInt): TModalResult;

implementation


uses LCLProc, LCLIntf;


const
  EmptyCaption = '';
  CBSPacer = 8;

  MsgDlgBtnToModalResult: Array[TMsgDlgBtn] of TModalResult = (mrYes, mrNo, mrOK, mrCancel, mrAbort,
        mrRetry, mrIgnore, mrAll, mrNoToAll, mrYesToAll, mrNone, mrCancel);

  {
  //Dutch captions
  nl_sMtWarning = 'Waarschuwing';
  nl_sMtError = 'Fout';
  nl_sMtInformation  = 'Informatie';
  nl_sMtConfirmation = 'Bevestiging';
  nl_sMtCustom = 'Eigen';

  nl_sMbYes          = '&Ja';
  nl_sMbNo           = '&Nee';
  nl_sMbOK           = '&OK';
  nl_sMbCancel       = 'Annuleer';
  nl_sMbAbort        = 'Afbreken';
  nl_sMbRetry        = '&Opnieuw';
  nl_sMbIgnore       = '&Negeer';
  nl_sMbAll          = '&Alles';
  nl_sMbNoToAll      = 'Nee op alles';
  nl_sMbYesToAll     = 'Ja op &alles';
  nl_sMbHelp         = '&Help';

  NlMsgDlgCaptions: Array[mtWarning..mtCustom] of String = (nl_sMtWarning, nl_sMtError,
        nl_sMtInformation, nl_sMtConfirmation, nl_sMtCustom);

  }


procedure CenterFormOver(AForm, CallerForm: TCustomForm);
var CX, CY, T, L, H, W: Integer;
begin
  if (CallerForm = nil) then CallerForm := Application.Mainform;
  if Assigned(CallerForm) then
  begin
    AForm.Position := poDesigned;
    //At this point, Top and Left are still 0 in the actual dialog, so we do the math ourselves
    W := Aform.Width;
    H := Aform.Height;
    CX := CallerForm.Left + (CallerForm.Width div 2);
    CY := CallerForm.Top + (CallerForm.Height div 2);
    {$IFDEF FPC}
    //Height returns ClientHeight;
    H := H + HeightCorrection;
    CY := CY + (HeightCorrection div 2);
    {$ENDIF}
    L := CX - (W div 2);
    T := CY - (H div 2);
    {$IFDEF FPC}
    //We corrected Height in calculations, now we must set it back
    H := H - HeightCorrection;
    {$ENDIF}
    AForm.SetBounds(L, T, W, H);
  end
  else
  begin
    AForm.Position := poScreenCenter;
  end;
end;


procedure KeepInView(AForm: TCustomForm);
//This proecdure ensures a form does not disappear from the desktop
//it is only applied to dialogs with poMainFormCenter !!
var DW, DH, T, L, H, W: Integer;
begin
  //debugln('KeepInView A');
  if Assigned(Application.MainForm) and (AForm.Position <> poScreenCenter) then
  begin
    W := Aform.Width;
    H := Aform.Height;
    L := AForm.Left;
    T := Aform.Top;
    DH := Screen.DesktopHeight;
    DW := Screen.DesktopWidth;
//    DebugLn('DW = ',DbgS(DW));
//    DebugLn('DH = ',DbgS(DH));
    //debugln('L = ',DbgS(L));
    //debugln('T = ',DbgS(T));
    //debugln('W = ',DbgS(W));
    //debugln('H = ',DbgS(H));
    //debugln('L+W = ',DbgS(L+W));
    //debugln('T+H = ',DbgS(T+H));
    if (L + W) > DW then
    begin
      //DbgOut('L: ',DbgS(L),' -> ');
      L := DW - W;
      //debugln(DbgS(L));
    end;
    if (L < 0) then L := 0;
    if (T + H) > DH then
    begin
      //DbgOut('T: ',DbgS(T),' -> ');
      T := DH - H;
      //debugln(DbgS(T));
    end;
    if (T < 0) then T := 0;
    //if (AForm.Left <> L) or (AForm.Top <> T) then
    if (L <> AForm.Left) or (T <> AForm.Top) then AForm.SetBounds(L, T, W, H);
  end;
  //debugln('KeepInView B');
end;





function ModalEscapeValue(Buttons : TMsgDlgButtons) : TModalResult;
begin
  If mbCancel in Buttons then
    Result := mrCancel
  else
    If mbNo in Buttons then
      Result := mrNo
  else
    If mbAbort in Buttons then
      Result := mrAbort
  else
    If mbIgnore in Buttons then
      Result := mrIgnore
  else
    If mbNoToAll in Buttons then
      Result := mrNoToAll
  else
    If mbYes in Buttons then
      Result := mrYes
  else
    If mbOk in Buttons then
      Result := mrOk
  else
    If mbRetry in Buttons then
      Result := mrRetry
  else
    If mbAll in Buttons then
      Result := mrAll
  else
    If mbYesToAll in Buttons then
      Result := mrYesToAll
  else
    Result:=mrCancel;
end;

{
procedure TranslateDlgCaptions(ADlg: TForm; ACaption: String; ADlgType: TMsgDlgType);
var
  Btn: TBitBtn;
  BtnCap: TCaption;
  i: Integer;
begin
  if (ACaption = EmptyCaption) and (ADlgType <> mtCustom) then  ADlg.Caption := NlMsgDlgCaptions[ADlgType];
  begin
    for i := 0 to ADlg.ComponentCount - 1 do
    begin
      if (ADlg.Components[i] is TBitBtn) then
      begin
        Btn := (ADlg.Components[i] as TBitBtn);
        BtnCap := Btn.Caption;
        if CompareText(BtnCap, rsMbYes) = 0 then Btn.Caption := nl_sMbYes else
          if CompareText(BtnCap, rsMbNo) = 0 then Btn.Caption := nl_sMbNo else
            if CompareText(BtnCap, rsMbOk) = 0 then Btn.Caption := nl_sMbOk else
              if CompareText(BtnCap, rsMbCancel) = 0 then Btn.Caption := nl_sMbCancel else
                if CompareText(BtnCap, rsMbAbort) = 0 then Btn.Caption := nl_sMbAbort else
                  if CompareText(BtnCap, rsMbRetry) = 0 then Btn.Caption := nl_sMbRetry else
                    if CompareText(BtnCap, rsMbIgnore) = 0 then Btn.Caption := nl_sMbIgnore else
                      if CompareText(BtnCap, rsMbAll) = 0 then Btn.Caption := nl_sMbAll else
                        if CompareText(BtnCap, rsMbNoToAll) = 0 then Btn.Caption := nl_sMbNoToAll else
                          if CompareText(BtnCap, rsMbYesToAll) = 0 then Btn.Caption := nl_sMbYesToAll else
                            if CompareText(BtnCap, rsMbHelp) = 0 then Btn.Caption := nl_sMbHelp;
      end;
    end;
  end;
end;
}


function MessageDlg(const aMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; CallerForm: TCustomForm): Integer;
begin
  Result := MessageDlg(EmptyCaption, aMsg, DlgType, Buttons, HelpCtx, mbYes, CallerForm);
end;

function MessageDlg(aMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; DefaultButton: TMsgDlgBtn;
  CallerForm: TCustomForm): Integer;
begin
  Result := MessageDlg(EmptyCaption, aMsg, DlgType, Buttons, HelpCtx, DefaultButton, CallerForm);
end;

function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; CallerForm: TCustomForm): Integer;
begin
  Result := MessageDlg(aCaption, aMsg, DlgType, Buttons, HelpCtx, mbYes, CallerForm);
end;

function MessageDlg(const aCaption, aMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; DefaultButton: TMsgDlgBtn;
  CallerForm: TCustomForm): Integer;
var Dlg: TForm;
    Res: TModalResult;
    UseDefaultBtn: Boolean;
    i: Integer;
    Btn: TBitBtn;
begin
  Dlg := CreateMessageDialog(AMsg, DlgType, Buttons);
  Try
    Dlg.HelpContext := HelpCtx;
    UseDefaultBtn := (DefaultButton in Buttons);
    if UseDefaultBtn then
    begin
      for i := 0 to Dlg.ComponentCount - 1 do
      begin
        if (Dlg.Components[i] is TBitBtn) then
        begin
          Btn := (Dlg.Components[i] as TBitBtn);
          if Btn.ModalResult = MsgDlgBtnToModalResult[DefaultButton] then
          begin
            Btn.Default := True;
            Dlg.ActiveControl := Btn;
            Break;
          end;
        end;
      end;
    end;
    if (ACaption <> EmptyCaption) then
      Dlg.Caption := ACaption;
    //if DoTranslateCaptions then TranslateDlgCaptions(Dlg, aCaption, DlgType);


    if (DefaultMsgDlgPosition = dpScreenCenter) then
      Dlg.Position := poScreenCenter
    else
      Dlg.Position := poDesigned;
    if (DefaultMsgDlgPosition <> dpScreenCenter) then
    begin
      if (DefaultMsgDlgPosition = dpMainFormCenter) then
        CenterFormOver(Dlg, Application.MainForm)
      else
        CenterFormOver(Dlg, CallerForm);
      KeepInView(Dlg);
    end;
    Res := Dlg.ShowModal;
    if (Res = -1) then
      Res := ModalEscapeValue(Buttons);
    Result := Res;
  Finally
    Dlg.Free;
    Dlg := nil;
  end;
end;

function MessageDlgPos(const aMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;
var Dlg: TForm;
    Res: TModalResult;
begin
  Dlg := CreateMessageDialog(AMsg, DlgType, Buttons);
  Try
    //if DoTranslateCaptions then TranslateDlgCaptions(Dlg, EmptyCaption, DlgType);
    Dlg.HelpContext := HelpCtx;
    Dlg.Left := X;
    Dlg.Top := Y;
    Res := Dlg.ShowModal;
    if (Res = -1) then
      Res := ModalEscapeValue(Buttons);
    Result := Res;
  Finally
    Dlg.Free;
    Dlg := nil;
  end;
end;

function ExtMessageDlg(const AMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckBoxCaption: String; out
  CheckBoxChecked: Boolean; HelpCtx: Longint; CallerForm: TCustomForm
  ): TModalResult; overload;
begin
  Result := ExtMessageDlg(EmptyCaption, AMsg, DlgType, Buttons, CheckBoxCaption,
                      CheckBoxChecked, HelpCtx, CallerForm);
end;

function ExtMessageDlg(const ACaption, AMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckBoxCaption: String;
  out CheckBoxChecked: Boolean; HelpCtx: Longint; CallerForm: TCustomForm = nil): TModalResult; overload;
var Dlg: TForm;
    Res: TModalResult;
    CB: TCheckBox;
begin
  Dlg := CreateMessageDialog(AMsg, DlgType, Buttons);
  CheckBoxChecked := False;
  Try
    Dlg.HelpContext := HelpCtx;
    if (ACaption <> EmptyCaption) then Dlg.Caption := ACaption;
    //if DoTranslateCaptions then TranslateDlgCaptions(Dlg, EmptyCaption, DlgType);
    if (DefaultMsgDlgPosition = dpScreenCenter) then
      Dlg.Position := poScreenCenter
    else
      Dlg.Position := poDesigned;
    //Create and add CheckBox
    CB := TCheckBox.Create(Dlg);
    CB.Parent := Dlg;
    CB.Checked := False;
    CB.Left := CBSpacer;
    CB.Width := Dlg.ClientWidth - (2*CBSpacer);
    Dlg.Height := Dlg.Height + CBSpacer + CB.Height;
    CB.Top := Dlg.ClientHeight - CB.Height - CBSpacer;
    CB.Caption := CheckBoxCaption;

    if (DefaultMsgDlgPosition <> dpScreenCenter) then
    begin
      if (DefaultMsgDlgPosition = dpMainFormCenter) then
        CenterFormOver(Dlg, Application.MainForm)
      else
        CenterFormOver(Dlg, CallerForm);
      KeepInView(Dlg);
    end;
    Res := Dlg.ShowModal;
    if (Res = -1) then
      Res := ModalEscapeValue(Buttons);
    Result := Res;
    CheckBoxChecked := CB.Checked;
  Finally
    Dlg.Free;
    Dlg := nil;
  end;
end;

function ExtMessageDlgPos(const ACaption, AMsg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckBoxCaption: String; out
  CheckBoxChecked: Boolean; HelpCtx: Longint; X, Y: LongInt): TModalResult;
var Dlg: TForm;
    Res: TModalResult;
    CB: TCheckBox;
begin
  Dlg := CreateMessageDialog(AMsg, DlgType, Buttons);
  CheckBoxChecked := False;
  Try
    Dlg.HelpContext := HelpCtx;
    if (ACaption <> EmptyCaption) then Dlg.Caption := ACaption;
    //if DoTranslateCaptions then TranslateDlgCaptions(Dlg, EmptyCaption, DlgType);
    Dlg.Position := poDesigned;
    //No we do not check the validity of X and Y, it's up to the user to keep them sane
    Dlg.Left := X;
    Dlg.Top := Y;
    //Create and add CheckBox
    CB := TCheckBox.Create(Dlg);
    CB.Parent := Dlg;
    CB.Checked := False;
    CB.Left := CBSpacer;
    CB.Width := Dlg.ClientWidth - (2*CBSpacer);
    Dlg.Height := Dlg.Height + CBSpacer + CB.Height;
    CB.Top := Dlg.ClientHeight - CB.Height - CBSpacer;
    CB.Caption := CheckBoxCaption;
    Res := Dlg.ShowModal;
    if (Res = -1) then
      Res := ModalEscapeValue(Buttons);
    Result := Res;
    CheckBoxChecked := CB.Checked;
  Finally
    Dlg.Free;
    Dlg := nil;
  end;
end;



Initialization
  DefaultMsgDlgPosition := dpCallerFormCenter;


end.
